/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.ModeledTree");

dojo.require("jazz.ui.Tree");
dojo.require("jazz.ui.tree.LinkItem");
dojo.require("jazz.ui.tree.SimpleItem");
dojo.require("jazz.ui.tree.IconItem");

(function() {

var LinkItem = jazz.ui.tree.LinkItem;
var SimpleItem = jazz.ui.tree.SimpleItem;
var IconItem = jazz.ui.tree.IconItem;

dojo.declare("jazz.ui.ModeledTree", jazz.ui.Tree, {
	// summary:
	//		A subclass of the basic jazz.ui.Tree which provides support for generating the
	//		tree content from a model.
	//
	//      NOTE: This is provisional API. Clients using this API do so with the
	//		awareness that this class may be removed or substantially changed at
	//		future times without warning.
	// description:
	//		The ModeledTree builds on the basic Tree functionality by allowing the consumer
	//		to define the tree structure by providing a javascript object that represents it.
	//		In addition, functions are provided to interact with the tree at the model level
	//		and to track the overall state of the tree.
	//
	//		The constructor accepts a single object which can have any of the properties the
	//		regular tree supports. In addition, there is one more property that is required
	//		for the ModeledTree to function. Failure to provide it will result in an error:
	//
	//			model: An array of objects representing the structure of the tree. Each item
	//				in the array represents a root in the tree. It can contain whatever
	//				properties are needed for use later when creating jazz.ui.tree.AbstractItem
	//				instances out of them. One thing to keep in mind when creating the model
	//				objects however, is that they are passed to the constructors of the item
	//				widgets, so any properties that collide with dojo properties/functions
	//				could cause unexpected behavior. One such property is the 'id'. Providing
	//				and 'id' property on the model will overwrite the id of the resulting dojo
	//				widget and could cause collisions. This should be avoided. The only reserved
	//				property otherwise is 'children'. This should be an array of model objects
	//				representing the children of the item.
	// example:
	//		var myModel = [{label: "Root 1", children: [{label: "Sub 1a", children:[{label: "Sub 1ai"}]},{label: "Sub 2a"}]},
	//					{label: "Root 2"},
	//					{label: "Root 3"}];
	//		var tree = new jazz.ui.ModeledTree({model: myModel});
	// since:
	//		0.7
	
	// PUBLIC API
	
	model: null,
	
	createTreeItem: function(modelObj) {
		// summary:
		//		Creates items from model objects.
		// description:
		//		This will be called by the ModeledTree when it needs to create instances of
		//		jazz.ui.tree.AbstractItem out of the tree's model objects. Consumers can override
		//		this to instantiate their own custom item types, but the default implementation
		//		will inspect the model object and pick an appropriate type based on its contents.
		//		If this is not overriden, one of the following will be used:
		//			-jazz.ui.tree.LinkItem (if the object has an 'href' or 'onclick')
		//			-jazz.ui.tree.IconItem (if the object has an 'iconClass')
		//			-jazz.ui.tree.SimpleItem (if the object doesn't meet the other conditions)
		// modelObj:
		//		The object in the model that represents the node to be created.
		if (modelObj.href || modelObj.onclick)
			return new LinkItem(modelObj);
		if (modelObj.iconClass)
			return new IconItem(modelObj);
		return new SimpleItem(modelObj);
	},
	
	expandTo: function(id, getId) {
		// summary:
		//		Expands the tree to the expose the node with the indicated id.
		// description:
		//		This function can be used to find a given node in the model and expand the tree
		//		to show that node. Because the tree must be traversed using the model, a function
		//		must be provided that knows how to determine the id of a given item based solely
		//		on its model object.
		//
		//		This means that all items in the tree must provide custom ids. The getId function
		//		of the items in the tree must be overriden so that the ids returned are consistent
		//		for a given model object the default implementation of jazz.ui.tree.AbstractItem
		//		just returns the widget id which can change across multiple loads and is not based
		//		on the model object, so it is not good enough for the purposes of this function.
		// id:
		//		The id of the desired node.
		// getId:
		//		A reference to a funciton that returns the id of an item based on its model object.
		//		The function should accept one parameter (the model object) and return the correct
		//		id.
		if (this._data[id]) {
			if (!this._data[id].item.isRoot())
				this._data[id].item.getParent().setExpanded(true);
			return;
		}
		var path = this._findPath(id, getId);
		if (path) {
			this._supressEvents = true;
			for (var i = 0; i < path.length; i++)
				this.getItem(path[i]).setExpanded(true);
			delete this._supressEvents;
		}
	},
	
	getModelObject: function(id) {
		// summary:
		//		Returns the model object for the given id.
		// description:
		//		Returns the model object for the tree item with the given id. For example,
		//		tree.getModelObject(treeItem.getId()) will return the model object that was used
		//		to generate the jazz.ui.tree.AbstractItem, treeItem.
		// id:
		//		The id of the tree item who's model object is needed.
		if (this._data[id])
			return this._data[id].modelObj;
	},
	
	getState: function() {
		// summary:
		//		Returns the expansion state of the tree.
		// description:
		//		Returns a javascript object that represents the expansion state of the entire tree.
		//		This is basically a map of ids of tree nodes that are expanded. This object can
		//		be passed to the restoreState function to return the tree to a previous state.
		//
		//		In order for the state to be valid across multiple loads of the ui, the items in the
		//		tree need to provide consistent ids by overriding the jazz.ui.tree.AbstractItem.getId
		//		function.
		return this._expanded;
	},
	
	onStateChange: function() {
		// summary:
		//		A hook to listen to expand/collapse events.
		// description:
		//		This function will be called any time a node is expanded or collapsed. This can be
		//		used to track the state of the tree and react to changes.
	},
	
	restoreState: function(state) {
		// summary:
		//		Restores the expand/collapse state of the tree based on the provided data.
		// description:
		//		This function can be used to restore the tree to a previous state by passing in the
		//		result of a previous call to getState.
		//
		//		In order for this to function across multiple loads of the ui, the items in the
		//		tree need to provide consistent ids by overriding the jazz.ui.tree.AbstractItem.getId
		//		function.
		this._supressEvents = true;
		this._expanded = state;
		this._doRestore(this.getChildren());
		delete this._supressEvents;
	},
	
	// INTERNAL PROPERTIES AND FUNCTIONS
	
	constructor: function(args) {
		this._data = {};
		this._expanded = {};
		this._childrenBuilt = {};
	},
	
	postCreate: function() {
		this.inherited(arguments);
		if (!this.model) {
			console.error("jazz.ui.ModeledTree is missing required parameter 'model'");
			return;
		}
		if (this.model) {
			for (var i = 0; i < this.model.length; i++) {
				this.addChild(this._buildItem(this.model[i]));
			}
		}
	},
	
	_isLeaf: function(item) {
		if (this._data[item.getId()]) {
			var obj = this._data[item.getId()].modelObj;
			return !(obj && obj.children && obj.children.length);
		}
		this.inherited(arguments);
	},
	
	_setExpanded: function(item, bool) {
		if (bool) {
			this._expanded[item.getId()] = true;
			var obj = this._data[item.getId()].modelObj;
			if (obj.children && !this._childrenBuilt[item.getId()]) {
				for (var i = 0; i < obj.children.length; i++) {
					item.addChild(this._buildItem(obj.children[i]));
				}
				this._childrenBuilt[item.getId()] = true;
			}
		} else {
			delete this._expanded[item.getId()];
		}
		this.inherited(arguments);
		if (!this._supressEvents)
			this.onStateChange();
	},
	
	_buildItem: function(modelObj) {
		var item = this.createTreeItem(modelObj);
		this._data[item.getId()] = {item: item, modelObj: modelObj};
		return item;
	},
	
	_doRestore: function(arr) {
		for (var i = 0; i < arr.length; i++) {
			var item = arr[i];
			item.setExpanded(this._expanded[item.getId()]);
			this._doRestore(item.getChildren());
		}
	},
	
	_findPath: function(id, getId, modelObjs) {
		if (!modelObjs)
			modelObjs = this.model;
		for (var i = 0; i < modelObjs.length; i++) {
			var obj = modelObjs[i];
			var thisId = getId(obj);
			if (thisId == id)
				return true;
			if (!obj.children)
				continue;
			var inPath = this._findPath(id, getId, obj.children);
			if (inPath) {
				if (inPath === true)
					return [thisId];
				inPath.splice(0,0,thisId);
				return inPath;
			}
		}
		return false;
	}
	
});

})();
