/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("jazz.app.ResourceLink");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.xml.DomParser");
dojo.require("dojox.data.dom");

dojo.require("jazz.app.proxy");
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.app.internal.loginUtil");
dojo.require("jazz.app.ua.internal.util");
dojo.require("jazz.app.internal.ResourceLinkAggregator");
dojo.require("jazz.util.html");

dojo.requireLocalization("jazz.app.internal", "ResourceLinkMessages");

var HoverPopup = jazz.ui.HoverPopup;
var TYPE_PREVIEW = "http://jazz.net/identifiers/presentations/preview";
var CSS_ARROW = "arrow";
var CSS_RL = "jazz-app-ResourceLink";
var CT_COMPACT_RENDERING = "application/x-jazz-compact-rendering";
var XMLNS_DC_TERMS = "http://purl.org/dc/terms/";
var XMLNS_JP = "http://jazz.net/xmlns/prod/jazz/presentation/1.0/";
var XMLNS_RDF = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";

var expandedResourceLink;
var defaultLazyFetch = false;

// use a single event handler for the web page since we'll have possibly many
// navigation links in a single page
var bodyClickListener = function(e){
	var target = e && e.target;
    if (!target) {
    	return; // early return
    }
	if (expandedResourceLink && !dojo.isDescendant(target, jazz.ui._getMasterPopup().domNode)) {
		expandedResourceLink.close();
	}
    if (target.tagName !== "A" || target.__jazzType !== "ResourceLink") {
    	return;
	}
    var widget = dijit.byId(target.getAttribute("widgetId"));
    if (widget) {
    	widget["onClick"](e);
    }
};
var bodyClickHandler = dojo.connect(dojo.body(), "onclick", bodyClickListener);
dojo.addOnUnload(function() {
	dojo.disconnect(bodyClickHandler);
});

dojo.declare("jazz.app.ResourceLink", dijit._Widget, {
    
	// summary:
	//		A hyperlink referencing a Jazz resource. We take the URL of the
	//		link's HREF attribute to determine the resource (Work Item, Plan,
	//		etc.) and then send back information to display a summary of the
	//		resource when the user hovers over the link. The summaries presented
	//		can have a small and large view depending on how much content the
	//		user wishes to see.
	//
	// parameters:
	//		uri: (String) The uri that will be used to find the available
	//				presentations. The URI can be relative ("/testURI") 
	//				or absolute ("http://localhost/testURI"). REQUIRED if 
	//				the given linkNode does not have a uri attached to 
	//				the HREF attribute.
	//		(optional) presentationUri: (String) A URL that will replace the
	//				link's HREF attribute, used to send a user to a different
	//				presentation of the given resource.
	//		(optional) label: (String) A string or HTML snippet that will be
	//				placed as the child node/inner text of the link/anchor node
	//				the Resource Link will be attached to.  If no srcNodeRef is 
	//				given, this argument is REQUIRED.
	//		(optional) retainLinkText: (boolean) If set to false, the text in the 
	//				srcNodeRef anchor node will change to the text of the compact 
	//				rendering's "dc:title" attribute. If set to true, the text of 
	//				the srcNodeRef anchor node will not change when the resource 
	//				link is created. By default, this parameter is set to false 
	//				and will change the text of the anchor node.
	//		(optional) iconNode: (DOM Node) The DOM node in the page to which the
	//				ResourceLink should append the icon specified by the compact
	//				rendering document
	//		(optional) srcNodeRef: (Anchor Node) The anchor node in the page
	//				that the Resource Link will be attached to. NOTE: if the uri
	//				parameter is not given, the href value of the given	node
	//				will be used in its place as the resource to reference.
	//		(optional) lazyFetch: (boolean) If set to true, the Resource Link will
	//				will not fetch the compact rendering document until the user
	//				hovers over the link.  Also, if this option is set to true,
	//				the "retainLinkText" parameter will automatically be set to true,
	//				and the "iconNode" parameter will be ignored.  By default,
	//				this parameter is set to false, and the compact rendering document
	//				will be aggressively fetched when the page loads.
	//
	// methods:
	//
	//		refresh: if the resource that the Resource Link is viewing has changed,
	//				one can call this method to retrieve the new compact rendering
	//				document and update the recource link hover with new data.
	//
	// usage:
	//		var navLink = new jazz.ui.ResourceLink({}, linkNode);
	//		-or-
    //		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//                  label: "Label for the Anchor Node"
	//				});
	//		-or-
	//		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//				},
	//				srcNodeRef);
	//		-or-
	//		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//					presentationUri: "anotherUrl"
	//				},
	//				srcNodeRef);
    //		-or-
	//		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//					presentationUri: "anotherUrl",
    //                  label: "New Label for the Anchor Node"
	//				},
	//				srcNodeRef);
    //		-or-
	//		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//					presentationUri: "anotherUrl",
    //                  label: "Label for the Anchor Node"
	//				});
	// since:
	//		0.7
	
	constructor: function () {
		this.messages = dojo.i18n.getLocalization("jazz.app.internal", "ResourceLinkMessages");
	},
	
	create: function(args, srcNodeRef){
		if (args.lazyFetch || (defaultLazyFetch && args.lazyFetch !== false)) {
			this._lazyFetch = true;
			this._iconNode = null;
			this._retainLinkText = true;
		} else {
			this._lazyFetch = false;
			this._retainLinkText = args.retainLinkText;
			this._iconNode = args.iconNode;
		}
		if ( !srcNodeRef && args.uri )
		{
			this.linkNode = document.createElement("A");
			
			if (args.uri.match(/http(?:s)?\:\/\//)) {
				this.uri = args.uri;
			}
			else {
				this.uri = new dojo._Url(document.baseURI, args.uri).toString();
			}
			
			if (args.presentationUri) 
				this.linkNode.href = args.presentationUri;
			else 
				this.linkNode.href = this.uri;
					
			this.linkNode.innerHTML = jazz.util.html.removeScript(args.label);
		}
		else if (srcNodeRef && srcNodeRef.tagName === "A" && !args.uri) {
			this.linkNode = srcNodeRef;
			var href = this.linkNode.href;
			if (href.match(/http(?:s)?\:\/\//)) {
				this.uri = href;
			}
			else {
				this.uri = (new dojo._Url(document.baseURI, href)).toString();
			}
			
			if (args.presentationUri) 
				this.linkNode.href = args.presentationUri;
			
			if ( args.label )
				this.linkNode.innerHTML = jazz.util.html.removeScript(args.label);
		}
		else if (srcNodeRef && srcNodeRef.tagName === "A" && args.uri) {
			this.linkNode = srcNodeRef;
			if (args.uri.match(/http(?:s)?\:\/\//)) {
				this.uri = args.uri;
			}
			else {
				this.uri = (new dojo._Url(document.baseURI, args.uri)).toString();
			}
			
			if (args.presentationUri) 
				this.linkNode.href = args.presentationUri;
			else 
				this.linkNode.href = this.uri;
				
			if ( args.label )
				this.linkNode.innerHTML = jazz.util.html.removeScript(args.label);
		}
		
		if (this.linkNode) {
			args.uri = this.uri;
			this.inherited("create", arguments, [args, this.linkNode]);
			if (!this._lazyFetch) {
				this._registerWithAggregator();
			} else {
				this._createLazyFetchHandler();
			}
		}
		else
			throw new Error("ResourceLink REQUIRES that either the uri parameter or a valid reference to an Anchor Node is given");
	},
	
	refresh: function(){
		var aggregator = jazz.app.internal.resourceLinkAggreagtor();
		aggregator.register(this.uri, dojo.hitch(this, this._refreshPresentation), dojo.hitch(this, this._errorRefresh), false);
	},
	
	_registerWithAggregator: function () {
		// Register this ResourceLink with the ResourceLinkAggregator.  The aggregator will retrieve
		// data for all ResourceLinks when the page is finished loading
		var aggregator = jazz.app.internal.resourceLinkAggreagtor();
		aggregator.register(this.uri, dojo.hitch(this, this._batchedPresentationHandler), dojo.hitch(this, this._errorPresentation), false);
	},
	
	_createLazyFetchHandler: function () {
		this._styleLink();
		//Create the HoverPopup
		this._popupLauncher = new HoverPopup({
			around: this.domNode,
			createContents: dojo.hitch(this, this._initLazyFetchPopup),
			onOpen: dojo.hitch(this, this._openPopup),
			onClose: dojo.hitch(this, this._closePopup),
			customClass: "jazz-app-ResourceHover"
		});
	},
	
	_initLazyFetchPopup: function (hoverPopup) {
		var dfd = jazz.client.xhrGet({
			url: this.uri,
			headers: {
				accept: CT_COMPACT_RENDERING
			}
		});
		dfd.addCallback(dojo.hitch(this, function (deferred, response) {
			if (deferred.ioArgs.xhr.getResponseHeader("Content-Type").substring(0,CT_COMPACT_RENDERING.length) === CT_COMPACT_RENDERING) {
				var crObj = this._parseCompactRenderingXML(dojox.xml.DomParser.parse(response));
				this._createPresentation(crObj);
			} else {
				dojox.data.dom.removeChildren(this._linkMetadata.loadingDiv);
				this._linkMetadata.loadingDiv.appendChild(document.createTextNode(this.messages.invalidCompactRendering));
				setTimeout(dojo.hitch(this, function () {
					this._disableHover();
				}), 1000);
			}
		}, dfd));
		dfd.addErrback(dojo.hitch(this, function (err) {
			dojox.data.dom.removeChildren(this._linkMetadata.loadingDiv);
			this._linkMetadata.loadingDiv.appendChild(document.createTextNode(this.messages.invalidCompactRendering));
			console.warn("ResourceLink Error (" + this.uri + "): " + err.message);
			setTimeout(dojo.hitch(this, function () {
				this._disableHover();
			}), 1000);
		}));
		this._linkMetadata = new net.jazz.app._LinkMetadata({
			smallHoverUrl: null,
			largeHoverUrl: null,
			icon: null,
			heading: null,
			showExpand: false,
			largeWidth: null,
			largeHeight: null,
			smallWidth: null,
			smallHeight: null,
			_popupLauncher: this._popupLauncher,
			linkNode: this.linkNode,
			uri: this.uri,
			token: this.token,
			oauthRequired: false,
			lazyFetch: true
		});
	
		return this._linkMetadata.domNode;
	},
	
	_parseCompactRenderingXML: function (crXml) {
		var crObj = {};
		var titleNodes = crXml.getElementsByTagNameNS("title", XMLNS_DC_TERMS);
		if (titleNodes.length > 0 && titleNodes[0].childNodes.length > 0) {
			crObj.title = titleNodes[0].childNodes[0].nodeValue;
		}
		var iconNodes = crXml.getElementsByTagNameNS("icon", XMLNS_JP);
		if (iconNodes.length > 0) {
			crObj.icon = iconNodes[0].getAttributeNS("resource", XMLNS_RDF);
		}
		var smallPreviewNodes = crXml.getElementsByTagNameNS("smallPreview", XMLNS_JP);
		if (smallPreviewNodes.length > 0) {
			crObj.smallPreview = smallPreviewNodes[0].getAttributeNS("resource", XMLNS_RDF);
			var pref_width = parseInt(smallPreviewNodes[0].getAttributeNS("preferred_width", XMLNS_JP));
			var pref_height = parseInt(smallPreviewNodes[0].getAttributeNS("preferred_height", XMLNS_JP));
			if (pref_width) {
				crObj.smallWidth = pref_width;
			}
			if (pref_height) {
				crObj.smallHeight = pref_height;
			}
		}
		var largePreviewNodes = crXml.getElementsByTagNameNS("largePreview", XMLNS_JP);
		if (largePreviewNodes.length > 0) {
			crObj.largePreview = largePreviewNodes[0].getAttributeNS("resource", XMLNS_RDF);
			var pref_width = largePreviewNodes[0].getAttributeNS("preferred_width", XMLNS_JP);
			var pref_height = largePreviewNodes[0].getAttributeNS("preferred_height", XMLNS_JP);
			if (pref_width) {
				crObj.largeWidth = pref_width;
			}
			if (pref_height) {
				crObj.largeHeight = pref_height;
			}
		}
		return crObj;
	},

	_batchedPresentationHandler: function(results) {
		//Style the link
		this._styleLink();
		
		if (results.oauthRequired){
			this.oauthRequired = true;
			this.token = results.token;
			
			//Create the HoverPopup
			this._popupLauncher = new HoverPopup({
				around: this.domNode,
				createContents: dojo.hitch(this, this._initPopup),
				onOpen: dojo.hitch(this, this._openPopup),
				onClose: dojo.hitch(this, this._closePopup),
				customClass: "jazz-app-ResourceHover",
				updateDimensions: dojo.hitch(this, this._updateDimensions)
			});
		
			return;
		} else {
			this.oauthRequired = false;
		}
		this._createPresentation(results);
	},
	
	_createPresentation: function(results){
		this.showHover = false;
		this.showExpand = false;
		
		if (results.title) {
			this.heading = jazz.util.html.removeScript(results.title);
			if (!this._retainLinkText) {
				this.linkNode.innerHTML = this.heading;
			}
		}
		
		if (results.icon) {
			this.icon = results.icon;
			if (this._iconNode) {
				var iconImg = document.createElement("img");
				iconImg.src = this.icon;
				this._iconNode.appendChild(iconImg);
			}
		}
		
		
		if (results.smallPreview) {
			this.smallHoverUrl = results.smallPreview;
			this.showHover = true;
		}
		
		if (results.largePreview) {
			this.largeHoverUrl = results.largePreview;
			if (this.showHover) 
				this.showExpand = true;
			else 
				this.showHover = true;
		}
		
		if (results.smallWidth && (Math.ceil(results.smallWidth) == Math.floor(results.smallWidth))) 
			this.smallWidth = parseInt(results.smallWidth);
		else
			this.smallWidth = 300;
		
		if (results.smallHeight && (Math.ceil(results.smallHeight) == Math.floor(results.smallHeight))) 
			this.smallHeight = parseInt(results.smallHeight);
		else
			this.smallHeight = 200;
		
		if (results.largeWidth && (Math.ceil(results.largeWidth) == Math.floor(results.largeWidth))) 
			this.largeWidth = parseInt(results.largeWidth);
		else
			this.largeWidth = Math.max(300, this.smallWidth);
		
		if (results.largeHeight && (Math.ceil(results.largeHeight) == Math.floor(results.largeHeight))) 
			this.largeHeight = parseInt(results.largeHeight);
		else
			this.largeHeight = Math.max(300, this.smallHeight);
		
		//Create the HoverPopup
		var oldLauncher = this._popupLauncher;
		this._popupLauncher = new HoverPopup({
			around: this.domNode,
			createContents: dojo.hitch(this, this._initPopup),
			onOpen: dojo.hitch(this, this._openPopup),
			onClose: dojo.hitch(this, this._closePopup),
			customClass: "jazz-app-ResourceHover",
			updateDimensions: dojo.hitch(this, this._updateDimensions)
		});
		if (oldLauncher) {
			if (oldLauncher._isOpen) {
				// do not need to close old launcher, will happen on open of new one
				this._popupLauncher.open();
			}
			oldLauncher.destroy();
		}
	},
	
	_errorPresentation: function (err) {
		console.warn("ResourceLink Error (" + this.uri + "): " + err.message);
		this._disableHover();
	},
	
	_refreshPresentation: function (results) {			
			var isNewPreview = false;
			var isNewSize = false;
			
			if (results.smallHoverUrl && results.smallHoverUrl !== this.smallHoverUrl) {
				this._linkMetadata.smallHoverUrl = results.smallHoverUrl;
				isNewPreview = true;
			}
			
			if (results.largeHoverUrl && results.largeHoverUrl !== this.largeHoverUrl) {
				this._linkMetadata.largeHoverUrl = results.largeHoverUrl;
				if (!this.smallHoverUrl){
					isNewPreview = true;	
				}				
			}
			
			if (results.icon && results.icon !== this.icon) {
				this._linkMetadata.icon = results.icon;
			}
				
			if (results.title && jazz.util.html.removeScript(results.title) !== this.heading) {
				this.heading = jazz.util.html.removeScript(results.title);
				if (!this._retainLinkText) {
					this.linkNode.innerHTML = this.heading;
				}
				this._linkMetadata.updateTitle(this.heading);	
			}
			
			if (results.smallWidth 
					&& (Math.ceil(results.smallWidth) == Math.floor(results.smallWidth)) 
					&& results.smallWidth !== this.smallWidth) {
				this._linkMetadata.smallWidth = parseInt(results.smallWidth);
				isNewSize = true;
			}
			
			if (results.smallHeight 
					&& (Math.ceil(results.smallHeight) == Math.floor(results.smallHeight))
					&&  results.smallHeight !== this.smallHeight ) {
				this._linkMetadata.smallHeight = parseInt(results.smallHeight);
				isNewSize = true;
			}
			
			if (results.largeWidth 
					&& (Math.ceil(results.largeWidth) == Math.floor(results.largeWidth))
					&&  results.largeWidth !== this.largeWidth ) {
				this._linkMetadata.largeWidth = parseInt(results.largeWidth);
				isNewSize = true;
			}
			
			if (results.largeHeight 
					&& (Math.ceil(results.largeHeight) == Math.floor(results.largeHeight))
					&&  results.largeHeight !== this.largeHeight ){ 
				this._linkMetadata.largeHeight = parseInt(results.largeHeight);
				isNewSize = true;
			}
			
			if (!isNewPreview){
				dojo.removeClass(this._linkMetadata.loadingImage, "hidden");
				dojo.addClass(this._linkMetadata.hoverIframe, "loadingIframe");
				this._linkMetadata.hoverIframe.src = this._linkMetadata.hoverIframe.src; 
			}
	},
	
	_errorRefresh: function (err) {
		console.warn("ResourceLink Refresh Error (" + this.uri + "): " + err.message);
	},
	
	_styleLink: function () {
		this.linkNode.__jazzType = "ResourceLink";
		dojo.addClass(this.linkNode, CSS_RL);
		var arrow = document.createElement("img");
		dojo.addClass(arrow, CSS_ARROW);
		arrow.src = dojo.moduleUrl("jazz.app", "internal/images/ResourceLink/rh-twistie.gif");
		this.linkNode.appendChild(arrow);
	},
	
	_disableHover: function () {		
		if (this._popupLauncher) {
			this._popupLauncher.close();
			this._popupLauncher.destroy();
		}
		for (var i=0; i < this.linkNode.childNodes.length; i++) {
			if (this.linkNode.childNodes[i].tagName == "IMG" && this.linkNode.childNodes[i].className == CSS_ARROW) {
				this.linkNode.removeChild(this.linkNode.childNodes[i]);
				break;
			}
		}
		dojo.removeClass(this.linkNode, CSS_RL);
	},
	
	_initPopup: function(hoverPopup){
		if( this.oauthRequired ){
			this._linkMetadata = new net.jazz.app._LinkMetadata({
				smallHoverUrl: null,
				largeHoverUrl: null,
				icon: null,
				heading: null,
				showExpand: false,
				largeWidth: null,
				largeHeight: null,
				smallWidth: null,
				smallHeight: null,
				_popupLauncher: this._popupLauncher,
				linkNode: this.linkNode,
				uri: this.uri,
				token: this.token,
				oauthRequired: this.oauthRequired				
			});
		
			return this._linkMetadata.domNode;
		}

		this._linkMetadata = new net.jazz.app._LinkMetadata({
			smallHoverUrl: this.smallHoverUrl,
			largeHoverUrl: this.largeHoverUrl,
			icon: this.icon,
			heading: this.heading,
			showExpand: this.showExpand,
			largeWidth: this.largeWidth,
			largeHeight: this.largeHeight,
			smallWidth: this.smallWidth,
			smallHeight: this.smallHeight,
			_popupLauncher: this._popupLauncher,
			linkNode: this.linkNode,
			uri: this.uri,
			oauthRequired: false
		});
		
		return this._linkMetadata.domNode;
	},
	
	_openPopup: function(){
		this._linkMetadata.open();
	},
	
	_updateDimensions: function(v) {
		this._linkMetadata.setHeightsAndWidths(v);
	},
	
	_closePopup: function() {
		this._popupLauncher.programmatic = false;
		if (this.showExpand) {
			this._linkMetadata._expandBtnDiv.className = "expansion";
		}
		expandedResourceLink = null;
		if (this._stopEventListener) {
			dojo.disconnect(this._stopEventListener);
			delete this._stopEventListener;
		}
	}
});
jazz.app.ResourceLink.setLazyFetch = function(bool) {
	defaultLazyFetch = bool;
};

//Creates te DOM elements that will hold the preview and presentation links displayed in the Hover Popup
dojo.declare("net.jazz.app._LinkMetadata", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("jazz.app", "internal/templates/ResourceLink.html"),
	
	constructor: function () {
		this.messages = dojo.i18n.getLocalization("jazz.app.internal", "ResourceLinkMessages");
	},
	
	postCreate: function(){
		dojo.body().appendChild(this.domNode);
		this.inherited(arguments);
		
		if (dojo.isIE){
			dojo.addClass(this.loadingImage, "hidden");
			dojo.removeClass(this.hoverIframe, "loadingIframe");
		}
		
		var loginLink = jazz.app.internal.loginUtil.insertLoginLink(this.oauthLoginDiv);
		this.connect(loginLink, "onclick", this.oauthLogin);
		
		this._closeButton.removeChild(this._closeButton.firstChild);
		this._expandButton.removeChild(this._expandButton.firstChild);
		
		if(this.oauthRequired){
			dojo.removeClass(this.oauthLoginDiv, "hideLogin");
		} else if (this.lazyFetch) {
			dojo.removeClass(this.loadingDiv, "hideLoading");
		} else{
			this._renderJsonPresentation();	
		}
	},
	
	/**
	* Toggles between displaying the small and large summary views.
	* @param {Object} e - mouse event
	*/
	displaySummary: function(e){
		dojo.stopEvent(e);
			
		if (this.showExpand) {
			if (!this._isFullDisplay) {
				this._isFullDisplay = true;
				this._popupLauncher.programmatic = "block";
				this._expandBtnDiv.className = "noExpansion";
				this.displayLargeSummary();
			}
			
			this._popupLauncher.position(this.linkNode);
		}
	},
	
	/**
	* Display the large content area.
	*/
	displayLargeSummary: function(){
		this._isFullDisplay = true;
		if (this.showExpand) 
			this._expandButton.className = "minimizeSummary";
			
		if (this.largeHoverUrl) {
			// update the iframe dimensions early to give an accurate size for the HoverPopup.position function
			this.updateIFrame();
			this.hoverIframe.src = this.largeHoverUrl;
		}
		expandedResourceLink = this._popupLauncher;
		this._stopEventListener = dojo.connect(dojo,"stopEvent",window,function(e) {
			if (e.type == "click")
				bodyClickListener(e);
		});
		// yield the calls to updateEllipsis and updateIFrame to ensure the UI is painted and positioning is done
		// run updateIFrame again after HoverPopup.position in case the effectiveWidth/Heights changes
		setTimeout(dojo.hitch(this,this.postPosition),0);
	},
	
	/**
	* Display the small content area.
	*/
	displaySmallSummary: function(){
		this._isFullDisplay = false;
		if (this.showExpand) 
			this._expandButton.className = "expandSummary";
		if (this.smallHoverUrl) {
			// update the iframe dimensions early to give an accurate size for the HoverPopup.position function
			this.updateIFrame();
			this.hoverIframe.src = this.smallHoverUrl;
		}
		expandedResourceLink = null;
		// yield the call to updateEllipsis and updateIFrame to ensure the UI is painted and positioning is done
		// run updateIFrame again after HoverPopup.position in case the effectiveWidth/Heights changes
		setTimeout(dojo.hitch(this, this.postPosition),0);
	},
	
	postPosition: function() {
		this.updateIFrame();
		this.updateEllipsis();
	},
	
	updateIFrame: function() {
		var w,h;
		if (this._isFullDisplay) {
			w = this.effectiveLargeWidth || this.largeWidth;
			h = this.effectiveLargeHeight || this.largeHeight;
		} else {
			w = this.effectiveSmallWidth || this.smallWidth;
			h = this.effectiveSmallHeight || this.smallHeight;
		}
		if (w) {
			this.loadingImage.style.width = this.hoverIframe.style.width = w + "px";
			if (dojo.isIE) {
				// IE needs explicit width on the divider and width 100% breaks it out the side
				// allow 4 px for margins on wrapper dix
				this.divider.style.width = (w - 4) + "px";
				this.bottomDivider.style.width = (w - 4) + "px";
			}
		}
		if (h) {
			this.loadingImage.style.height = this.hoverIframe.style.height = h + "px";
		}
	},
	
	updateEllipsis: function() {
		// hide the ellipsis first in case there is just enough room without it
		dojo.addClass(this.ellipsisPlaceholder, "hidden");
		// check if the titleNode is taller than the titleBar
		// subtract half the titlebar height in comparison to account for difference in single line sizes
		if (this.titleNode.offsetHeight - (this.titleBar.offsetHeight/2) > this.titleBar.offsetHeight) {
			// if the title node is too tall, it is wrapping, so show the ellipsis
			dojo.removeClass(this.ellipsisPlaceholder, "hidden");
			dojo.removeClass(this.ellipsis, "hidden");
			this.ellipsis.style.left = this.titleNode.offsetWidth + "px";
			this.titleNode.title = this.tooltipText;
		} else {
			this.titleNode.title = "";
			dojo.addClass(this.ellipsis, "hidden");
		}
	},
	
	/**
	* Method that determines if the available presentations should be shown or not in the Hover Popup when it appears.
	*/
	open: function(){
		if (this.smallHoverUrl) 
			this.displaySmallSummary();
		else 
			this.displayLargeSummary();
	},
	
	/**
	* Close the hover popup.
	* @param {Object} e - a mouse event.
	*/
	close: function(e){
		dojo.stopEvent(e);
		this._popupLauncher.close();
	},
	
	_contentLoaded: function(e){
		//Not checking for IE becaues there is a bug in IE where iframe contents are not shown
		//when "visibility: hidden" is removed from the iframe's CSS attribute
		if ( e.type === "load"){
			dojo.addClass(this.loadingImage, "hidden");
			dojo.removeClass(this.hoverIframe, "loadingIframe");
		}
	},	
	/**
	 * Access the compact rendering document on an outside server by logging in with
	 * Oauth.
	 * @param {Object} evt - mouse event
	 */
	oauthLogin: function(evt){
		dojo.stopEvent(evt);
		// handleAs "xml" doesn't work if the content-type isn't "text/xml"
		// In this case, the content type should be CT_COMPACT_RENDERING
		// so we handleAs "text" and parse the XML document in the handler
		var dfd = jazz.client.xhrGet({
			url: this.uri,
			headers: {
				"Accept": CT_COMPACT_RENDERING
			},
			handleAs: "text"
		});
		dfd.addCallback(dojo.hitch(this, this.renderOauthResponse));
		dfd.addErrback(dojo.hitch(this, this.renderOauthError));
	},

	/**
	 * This method breaks down the values in the JSON object acquired from the CompactRendering servlet
	 * that are then passed to properties of the _LinkMetaData class.
	 */
	_renderJsonPresentation: function(){
		this.updateTitle(this.heading);
		
		if (this.smallHoverUrl) {
			this._smallHoverHeadDfd = jazz.client.xhrHead({
				url: this.smallHoverUrl
			});
			this._smallHoverHeadDfd.addCallback(dojo.hitch(this, function (xhr) {
				if (xhr.status >= 400) {
					console.warn("Response code " + xhr.status + " for small hover (" + this.smallHoverUrl + ")");
					this.smallHoverUrl = dojo.moduleUrl("jazz.app", "internal/html/SmallPreviewMissing.html");
				} 
				if (!this._isFullDisplay) {
					this.hoverIframe.src = this.smallHoverUrl;
				}
			}, this._smallHoverHeadDfd.ioArgs.xhr));
		}
		if (this.largeHoverUrl) {
			this._largeHoverHeadDfd = jazz.client.xhrHead({
				url: this.largeHoverUrl
			});
			this._largeHoverHeadDfd.addCallback(dojo.hitch(this, function (xhr) {
				if (xhr.status >= 400) {
					console.warn("Response code " + xhr.status + " for large hover (" + this.largeHoverUrl + ")");
					this.largeHoverUrl = dojo.moduleUrl("jazz.app", "internal/html/LargePreviewMissing.html");
				}
				if (this._isFullDisplay) {
					this.hoverIframe.src = this.largeHoverUrl;
				}
			}, this._largeHoverHeadDfd.ioArgs.xhr));
		}
			
		//IE does not like <img> without a source value, so we hide the image unless there is a src value
		if (this.icon) {
			this.iconImg.src = this.icon;
			this.iconImg.className = "icon";
		}
		
		if (this.showExpand) {
			this._isFullDisplay = false;
			this._expandBtnDiv.className = "expansion";
		}
	},
	
	/**
	 * After we authenticate with OAuth, this method has the ResourceLinkAggregator resend
	 * an xhr request for all ResourceLinks that access the same server.
	 * @param {Object} results
	 */
	renderOauthResponse: function(results) {
		var aggregator = jazz.app.internal.resourceLinkAggreagtor();
		aggregator.resendOauth(this.token);
	},
	
	updateTitle: function(heading) {
		var index, endIndex;
		var title = [];
		var tooltip = [];
		var segments = [];
		var entities = [];
		// find any entities and split the input around them (to avoid breaking them up with <wbr/>s)
		while ((index = heading.indexOf("&")) != -1) {
			segments.push(heading.substring(0,index));
			heading = heading.substring(index);
			endIndex = heading.indexOf(";");
			entities.push(heading.substring(0, endIndex + 1));
			heading = heading.substring(endIndex + 1);
		}
		segments.push(heading);
		// for each segment, find html tags and preserve them, but split everything else with <wbr/>s
		for (var i = 0; i < segments.length; i++) {
			var currentSegment = [];
			heading = segments[i];
			if (heading && heading.length > 0) {
				while ((index = heading.indexOf("<")) != -1) {
					var str = heading.substring(0, index);
					if (str.length > 0) {
						tooltip.push(str);
						currentSegment.push(str.split("").join("<wbr/>").replace(/ <wbr\/>/g,"&nbsp;"));
					}
					endIndex = heading.indexOf(">");
					currentSegment.push(heading.substring(index, endIndex + 1));
					heading = heading.substring(endIndex + 1);
				}		
				tooltip.push(heading);
				currentSegment.push(heading.split("").join("<wbr/>").replace(/ <wbr\/>/g,"&nbsp;"));
			}
			currentSegment = currentSegment.join("");
			if (currentSegment.length > 0)
				title.push(currentSegment);
			if (i < entities.length) {
				tooltip.push(entities[i]);
				title.push(entities[i]);
			}
		}
		this.titleNode.innerHTML = title.join("<wbr/>");
		this.titleNode.href = this.linkNode.href;
		this.footerLink.href = this.linkNode.href;
		// decode any entities for the tooltip text
		var node = document.createElement("div");
		node.innerHTML = tooltip.join("");
		tooltip = [];
		node = node.firstChild;
		while (node) {
			if (node.nodeType == 3)
				tooltip.push(node.nodeValue);
			node = node.nextSibling;
		}
		this.tooltipText = tooltip.join("");
	},
	setHeightsAndWidths: function (v) {
		// HoverPopup will treat a width greater than viewport - 60 as viewport - 60
		// update the effective widths so the iframe is sized according to the max
		// account for the 5 px horizontal padding around the iframe
		if (this.smallWidth)
			this.effectiveSmallWidth = (v.w - 60 < this.smallWidth + 5) ? v.w - 65 : this.smallWidth;
		if (this.largeWidth)
			this.effectiveLargeWidth = (v.w - 60 < this.largeWidth + 5) ? v.w - 65 : this.largeWidth;
		
		// HoverPopup will treat a height greater than viewport - 30 as viewport - 30
		// update the effective widths so the iframe is sized according to the max
		// account for the 60 px vertical padding/content around the iframe
		if (this.smallHeight)
			this.effectiveSmallHeight = (v.h - 30 < this.smallHeight + 60) ? v.h - 90 : this.smallHeight;
		if (this.largeHeight)
			this.effectiveLargeHeight = (v.h - 30 < this.largeHeight + 60) ? v.h - 90 : this.largeHeight;
		
		if (this.effectiveSmallWidth && this.effectiveSmallWidth + 5 > 500) {
			this._popupLauncher.maxWidth = this.effectiveSmallWidth + 5;
		}
		if (this.effectiveSmallHeight && this.effectiveSmallHeight + 60 > 300) {
			this._popupLauncher.maxHeight = this.effectiveSmallHeight + 60;
		}
		if (this.effectiveLargeWidth && this.effectiveLargeWidth + 5 > 500 && (this.effectiveLargeWidth > this.effectiveSmallWidth || !this.effectiveSmallWidth)) {
			this._popupLauncher.maxWidth = this.effectiveLargeWidth + 5;
		}
		if (this.effectiveLargeHeight && this.effectiveLargeHeight + 60 > 300 && (this.effectiveLargeHeight > this.effectiveSmallHeight || !this.effectiveSmallHeight)) {
			this._popupLauncher.maxHeight = this.effectiveLargeHeight + 60;
		}
	}
});

})();