/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.StyledTable");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

(function() {

jazz.ui.styledTable = {
	TrimStyles: {
		GOLD: 0,
		BLUE: 1,
		GREY: 2
	}
};
var TrimStyles = jazz.ui.styledTable.TrimStyles;

dojo.declare("jazz.ui.StyledTable", [dijit._Widget, dijit._Templated], {
	// summary:
	//		A simple widget that creates a table with common Jazz web ui styles.
	//
	//      NOTE: This is provisional API. Clients using this API do so with the
	//		awareness that this class may be removed or substantially changed at
	//		future times without warning.
	// description:
	//		The StyledTable provides 3 common trim styles used throughout Jazz web UIs: gold, blue
	//		and grey. In addition, all styles support rendering an arrow on the current sort column
	//		as well as tracking the current sorting settings. A callback can be provided to react to
	//		the user's header clicks and ensure the sorting remains in sync with the table. By
	//		default the table renders a border above the columns that matches the trim style in
	//		color, though this can be disabled.
	//
	//		The widget populates the table heading and applies css to style the rows appropriatly.
	//		The consumer is reponsible for actually populating the table by accessing the <tbody>
	//		element through the tbody attribute of the widget (e.g. myTable.tbody.appendChild(myRow)).
	//
	//		Basic CSS is applied to the table to give it the style specified, but there is a well
	//		defined css class on the root of the table to allow consumers to size it according to
	//		needs. This class is jazz-ui-StyledTable. Styles can also be applied to the td and tr
	//		elements under jazz-ui-StyledTable to adjust paddings of the cells etc.
	// columns (required):
	//		An array of objects to represent each of the columns in the table. Must be passed in the
	//		contructor and cannot be changed during the lifecycle of the widget. Each object in the
	//		array supports the following properties:
	//			label (required):
	//				The label for the heading of the table column.
	//			width (optional):
	//				A string which, if specified, will be applied directly to the th for the column
	//				under domNode.width (note, this is the table element width, NOT css width).
	//			className (optional):
	//				A string which, if specified, will be added to the table headings as a class to
	//				allow custom styling for the column.
	//			sortable (optional):
	//				A boolean to indicate if the table can be sorted by this column. If true, the
	//				heading of the column will have a hover style to indicate the sortability. Clicks
	//				on sortable columns will update the rendering in the header to indicate the
	//				current sort order. They will also cause the sortBy and sortByAscending attributes
	//				to be updated followed by a call to handleSort.
	// trim (optional):
	//		An integer representing one of the predefined table styles. Must be one of the constants
	//		defined in jazz.ui.styledTable.TrimStyles. (default TrimStyles.GREY)
	// topBorder (optional):
	//		A boolean to indicate if a border should be painted above the table heading. (default true)
	// sortBy (optional):
	//		An integer representing the index of the current sort column (based on the array indeces of
	//		the columns array). Can be passed in the constructor to indicate the default sort order,
	//		but after that it becomes a read only parameter to be read from handleSort to react to
	//		changes in the sort order.
	// sortByAscending (optional):
	//		A boolean indicating if the current sort is ascending or decending. Will be toggled when the
	//		user clicks on the currently sorted column heading. Like sortBy, this can be set in the
	//		constructor but is then read only for the remainder of the widget's lifecycle. (default true)
	// since:
	//		0.7.1

	columns: null,
	
	trim: TrimStyles.GREY,
	topBorder: true,
	
	sortBy: null,
	sortByAscending: true,

	clearBody: function() {
		// summary:
		//		Removes all the contents of the table's body.
		if (dojo.isIE) {
			// replace the tbody and update the reference
			var newBody = document.createElement("tbody");
			this.domNode.replaceChild(newBody, this.tbody);
			this.tbody = newBody;
			// apply the trim style to the new body
			this._applyStyle();
		} else {
			this.tbody.innerHTML = "";
		}
	},

	handleSort: function() {
		// summary:
		//		A callback to allow consumers to react to changes in the table sorting.
		// description:
		//		To be overriden by consumers.
		//
		//		Will be called when the user clicks the heading of a sortable column. Before the
		//		function is invoked, the sortBy and sortByAscending variables will be updated so
		//		the consumer can react to the new settings accordingly.
	},

	setTrim: function(trim) {
		// summary:
		//		Updates the trim style of the table.
		// trim:
		//		One of the constants defined in jazz.ui.styledTable.TrimStyles.
		if (this.trim == trim)
			return;
		dojo.removeClass(this.tbody, this._getStyleClass(this.trim));
		dojo.removeClass(this._thead, this._getStyleClass(this.trim));
		this.trim = trim;
		this._applyStyle();
	},
	
	// INTERNAL PROPERTIES AND FUNCTIONS
	
	templatePath: dojo.moduleUrl("jazz.ui", "templates/StyledTable.html"),
	
	postCreate: function() {
		this._applyStyle();
		if (!this.topBorder)
			this._headerLine.style.display = "none";
		this._sortNodes = [];
		for (var i = 0; i < this.columns.length; i++) {
			var heading = this.columns[i];
			var cell = document.createElement("th");
			dojo.addClass(cell,"headerColumn");
			if (heading.className)
				dojo.addClass(cell, heading.className);
			if (heading.width) {
				cell.width = heading.width;
			}
			var div = document.createElement("div");
			div.innerHTML = heading.label.replace(/ /g,"&nbsp;");
			cell.appendChild(div);
			if (heading.sortable) {
				this._sortNodes[i] = cell;
				dojo.addClass(cell,"sortableColumn");
				var arrow = document.createElement("div");
				arrow.className = "arrow";
				cell.appendChild(arrow);
				this.connect(cell,"onclick",dojo.hitch(this,this._onSort,i));
				if (this.sortBy == i) {
					this._sortNode = cell;
					dojo.addClass(cell, this.sortByAscending ? "sortAscending" : "sortDescending");
				}
			}
			this._headerRow.appendChild(cell);
			if (this.topBorder) {
				var line = document.createElement("th");
				dojo.addClass(line,"headerColumnLine");
				if (heading.className)
					dojo.addClass(line, heading.className);
				if (heading.width) {
					line.width = heading.width;
				}
				this._headerLine.appendChild(line);
			}
		}
	},
	
	_onSort: function(index) {
		if (this._sortNode)
			dojo.removeClass(this._sortNode, this.sortByAscending ? "sortAscending" : "sortDescending");
		this._sortNode = this._sortNodes[index];
		if (index == this.sortBy)
			this.sortByAscending = !this.sortByAscending;
		else {
			this.sortBy = index;
			this.sortByAscending = true;
		}
		dojo.addClass(this._sortNode, this.sortByAscending ? "sortAscending" : "sortDescending");
		this.handleSort();
	},
	
	_applyStyle: function() {
		dojo.addClass(this.tbody, this._getStyleClass(this.trim));
		dojo.addClass(this._thead, this._getStyleClass(this.trim));
	},
	
	_getStyleClass: function(trim) {
		switch (trim) {
			case TrimStyles.GREY:
				return "table-grey";
			case TrimStyles.GOLD:
				return "table-gold";
			case TrimStyles.BLUE:
				return "table-blue";
		}
	}

});

})();
