/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.util.date");

dojo.require("dojo.date");
dojo.require("dojo.date.locale");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.requireLocalization("jazz.util", "dateMessages");

(function() {

jazz.util.date = {
	
	// summary:
	//		Provides supplemental date/time-related functions not provided by dojo.
	//
	//      NOTE: This is provisional API. Clients using this API do so with the
	//		awareness that this class may be removed or substantially changed at
	//		future times without warning.
	// usage notes:
	//		- access this object via 'static' methods (e.g.
	//		  jazz.util.date.formatDif())
	//		- do not instantiate
	// since:
	//		0.7
		
	/* string */ formatDif: function(/* Date */ date, /* boolean */ showAbsoluteDate, /* boolean */ showRelativeDate) {
		// summary:
		//		Formats the given date into a string indicating the difference compared
		//		to the current time (past or future). At least one of showAbsoluteDate
		//		and showRelativeDate is required.
		//
		//		Examples of calling this method with true supplied for
		//		showAbsoluteDate and showRelativeDate:
		//
		//		Now + 5 Seconds: 12:15 PM (In 1 minute)
		//		Now - 5 Minutes: 12:10 PM (5 minutes ago)
		//		Now + 5 Hours: 5:15 PM (In 5 hours)
		//		Now + 1 Day: Sep 30 12:26 PM (Tomorrow)
		//		Now - 2 Days: Sep 27 12:26 PM (2 days ago)
		//		Now + 1 Week: Oct 06 12:26 PM (Next Week)
		//		Now + 3 Weeks: Oct 20 12:26 PM
		//		Now - 1 Year: Sep 29 2005 12:26 PM
		//		
		//		Examples for passing false for showAbsoluteDate:
		//		
		//		Now + 5 Seconds: In 5 minutes
		//		Now - 2 Hours: 2 hours ago
		//	date:
		//		the Date to compare to the current time.
		//	showAbsoluteDate:
		//		if true, the specific date is made clear in the result
		//	showRelativeDate:
		//		if true, a rough approximation of the time difference is indicated.
		//	return:
		//		a string representing the given date in the specified format

		var str= "";

		if (!date)
			return "";

		var fgCalBase = new Date();
		var fgCalActual = date;
		
		var baseDay= _getDayOfYear(fgCalBase);
		var actualDay= _getDayOfYear(fgCalActual);

		var baseWeek= _getWeekOfYear(fgCalBase);
		var actualWeek= _getWeekOfYear(fgCalActual);

		var baseYear= fgCalBase.getFullYear();
		var actualYear= fgCalActual.getFullYear();
		
		/* Same Day */
		if (baseDay == actualDay && baseYear == actualYear) {
			if (showAbsoluteDate)
				str += dojo.date.locale.format(date, { selector: "date", formatLength: "medium" }) + " " + dojo.date.locale.format(date, { selector: "time", formatLength: "short" });

			if (showAbsoluteDate && showRelativeDate)
				str += " ";

			if (showRelativeDate)
				str += (showAbsoluteDate ? "(" : "") + _getReadableTimeDif(date) + (showAbsoluteDate ? ")" : "");
		}

		/* Yesterday */
		else if ((baseDay - actualDay) == 1 && baseYear == actualYear) {
			if (showAbsoluteDate)
				str += dojo.date.locale.format(date, { selector: "date", formatLength: "medium" }) + " " + dojo.date.locale.format(date, { selector: "time", formatLength: "short" });

			if (showAbsoluteDate && showRelativeDate)
				str += " ";

			if (showRelativeDate)
				str += (showAbsoluteDate ? "(" : "") + _messages.yesterday + (showAbsoluteDate ? ")" : "");
		}

		/* Tomorrow */
		else if ((actualDay - baseDay) == 1 && baseYear == actualYear) {
			if (showAbsoluteDate)
				str += dojo.date.locale.format(date, { selector: "date", formatLength: "medium" }) + " " + dojo.date.locale.format(date, { selector: "time", formatLength: "short" });

			if (showAbsoluteDate && showRelativeDate)
				str += " ";

			if (showRelativeDate)
				str += (showAbsoluteDate ? "(" : "") + _messages.tomorrow + (showAbsoluteDate ? ")" : "");
		}

		/* Same Week */
		else if (baseWeek == actualWeek && baseYear == actualYear) {
			if (showAbsoluteDate)
				str += dojo.date.locale.format(date, { selector: "date", formatLength: "medium" }) + " " + dojo.date.locale.format(date, { selector: "time", formatLength: "short" });

			if (showAbsoluteDate && showRelativeDate)
				str += " ";

			if (showRelativeDate) {
				var dayStr;
				var dayDif= actualDay - baseDay;
				if (dayDif * dayDif == 1) 
					dayStr = (dayDif > 0) ? _messages.inDay : _messages.pastDay;
				else
					dayStr = (dayDif > 0) ? dojo.string.substitute(_messages.inDays, [dayDif]) : dojo.string.substitute(_messages.pastDays, [-dayDif]);

				str += (showAbsoluteDate ? "(" : "") + dayStr + (showAbsoluteDate ? ")" : "");
			}
		}

		/* Last Week */
		else if ((baseWeek - actualWeek) == 1 && baseYear == actualYear) {
			if (showAbsoluteDate)
				str += dojo.date.locale.format(date, { selector: "date", formatLength: "medium" }) + " " + dojo.date.locale.format(date, { selector: "time", formatLength: "short" });

			if (showAbsoluteDate && showRelativeDate)
				str += " ";

			if (showRelativeDate)
				str += (showAbsoluteDate ? "(" : "") + _messages.lastWeek + (showAbsoluteDate ? ")" : "");
		}

		/* Next Week */
		else if ((actualWeek - baseWeek) == 1 && baseYear == actualYear) {
			if (showAbsoluteDate)
				str += dojo.date.locale.format(date, { selector: "date", formatLength: "medium" }) + " " + dojo.date.locale.format(date, { selector: "time", formatLength: "short" });

			if (showAbsoluteDate && showRelativeDate)
				str += " ";

			if (showRelativeDate)
				str += (showAbsoluteDate ? "(" : "") + _messages.nextWeek + (showAbsoluteDate ? ")" : "");
		}

		/* Everything else */
		else {
			str += dojo.date.locale.format(date, { selector: "date", formatLength: "medium" });
			if (showAbsoluteDate) {
				str += " " + dojo.date.locale.format(date, {selector: "time", formatLength: "short"});
			}
		}

		return str;
	},
	
	/* string */ formatDuration: function(/* int */ duration) {
		// summary:
		//		Formats the given number of milliseconds into an easy to read,
		//		approximate duration. e.g.:
		//
		//		5000: 5 seconds
		//		200000: 3 minutes, 20 seconds
		//		99000111: 1 day, 4 hours
		//	duration:
		//		the number of milliseconds for the duration
		//	return:
		//		a string representing the given date in the specified format

		var timeLeft = Math.round(duration / 1000);
		var seconds = timeLeft % 60;
		timeLeft = Math.floor(timeLeft / 60);
		if (timeLeft > 0) {
			var minutes = timeLeft % 60;
			timeLeft = Math.floor(timeLeft / 60);
			if (timeLeft > 0) {
				var hours = timeLeft % 24;
				timeLeft = Math.floor(timeLeft / 24);
				if (timeLeft > 0) {
					var days = timeLeft;
				}
			}
		}
		var result = _messages.lessThanSec;
		if (seconds || minutes || hours || days) {
			if (days)
				result = days == 1 ? _messages.day : dojo.string.substitute(_messages.days, [ days ]);
			if (hours)
				result = (days ? result + ", " : "") + (hours == 1 ? _messages.hour : dojo.string.substitute(_messages.hours, [ hours ]));
			if (minutes)
				result = (days || hours ? result + ", " : "") + (minutes == 1 ? _messages.min : dojo.string.substitute(_messages.mins, [ minutes ]));
			if (seconds)
				result = (days || hours || minutes ? result + ", " : "") + (seconds == 1 ? _messages.sec : dojo.string.substitute(_messages.secs, [ seconds ]));
		}
		return result;
	}	
};

//
// Internal; do not call or reference
//

var _messages = dojo.i18n.getLocalization("jazz.util", "dateMessages");

var _getDayOfYear = function(/* Date */ date){
	return dojo.date.difference(new Date(date.getFullYear(), 0, 1), new Date(date.getFullYear(), date.getMonth(), date.getDate())) + 1;
};

var _getReadableTimeDif = function(/* Date */ date) {
	if (!date)
		return "";

	var now= new Date().getTime();
	var then= date.getTime();
	var dif= now - then;

	var second= 1000;
	var minute= second * 60;
	var hour= minute * 60;
	var day= hour * 24;

	var minuteIntervals= [ 1, 2, 3, 5, 10, 15, 20, 30, 45, 60 ];
	var hourIntervals= [ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24 ];

	/* Event is Now (dif < 60 Seconds) */
	if (dif > -minute && dif < minute)
		return _messages.now;

	/* Event is in the Future */
	else if (dif < 0) {
		dif*= -1;

		/* Minutes */
		if (dif < minute * 2)
			return _messages.inMin;

		for (var i= 2; i < minuteIntervals.length; i++) {
			if (dif < minuteIntervals[i] * minute)
				return dojo.string.substitute(_messages.inMins, [ minuteIntervals[i-1] ]);
		}

		/* Hours */
		if (dif < hour * 2)
			return _messages.inHour;

		for (var i= 1; i < hourIntervals.length; i++) {
			if (dif < hourIntervals[i] * hour)
				return dojo.string.substitute(_messages.inHours, [ hourIntervals[i-1] ]);
		}

		if (dif < 2 * day) {
			return _messages.inDay;
		}
		return _messages.inSeveralDays;
	}

	/* Event is in the Past */
	else {

		/* Minutes */
		if (dif < minute * 2)
			return _messages.pastMin;

		for (var i= 2; i < minuteIntervals.length; i++) {
			if (dif < minuteIntervals[i] * minute)
				return dojo.string.substitute(_messages.pastMins, [ minuteIntervals[i-1] ]);
		}

		/* Hours */
		if (dif < hour * 2)
			return _messages.pastHour;

		for (var i= 2; i < hourIntervals.length; i++) {
			if (dif < hourIntervals[i] * hour)
				return dojo.string.substitute(_messages.pastHours, [ hourIntervals[i-1] ]);
		}

		if (dif < 2 * day) {
			return _messages.pastDay;
		}
		return _messages.pastSeveralDays;
	}
};

var _getWeekOfYear = function(/* Date */ date){
	var firstDayOfYear = new Date(date.getFullYear(), 0, 1).getDay();
	var adj = (firstDayOfYear + 7) % 7;
	var week = Math.floor((_getDayOfYear(date) + adj - 1) / 7);
	if(firstDayOfYear == 0){ week++; }
	return week;
};

})();
